/////////////////////////////////////////////////////////////////////////////
//File     :  TankGUI.java
//Author(s):  Chris (Episthene) Baillargeon
//Description: Main GUI to choose files, set the options and start the Tank Extractor
//				THe tank Extractor can be started independantly of the GUI if used as a library.

//copyright (C) 2005 Chris (Episthene) Baillargeon

//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//http://www.gnu.org/licenses/gpl.html

//----------------------------------------------------------------------------
//$Revision: 1.1               $Date: 2005/10/09  
//----------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////


package epi_extractor;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import epi_tools.*;
import java.awt.datatransfer.*;
import java.util.concurrent.*;

public class TankGUI extends JPanel implements ActionListener, ITankCaller {
									

/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
private  JButton buttonSelectTank;
private  JButton buttonSelectDirectory;
private  JButton buttonExtract;
private  JCheckBox checkLiquid;
private  JTextField textTankPath;
private  JTextField textDirectoryPath;
private  JLabel labelInfo;
private  JTextField progressStatus;
private  Container pane;
private  JFrame frame;
private  Epi_Config conf;
private  TankReader tankReader;
private JProgressBar progressBar;
private Timer timer;
private Future<String> futReturn;

private final int TIMER_DURATION = 200;
private final String WAITING_MESSAGE = "Waiting to Extract ...";

    public TankGUI() throws Exception {
    	
        conf = new Epi_Config();
        conf.loadConfigFile();
  
        pane = this;
    	pane.setLayout(new GridBagLayout());
        
        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.HORIZONTAL;

        //Label Info   
        labelInfo = new JLabel("Instructions", SwingConstants.LEFT);
        labelInfo.setText("<html>"+"1) Select the Tank to extract."+"<br>"+
        		  "2) Select extraction destination Folder (Where to extract the files)."+"<br>"+
				  "3) Check the box if you want to extract Liquid files. Note: If you extract a map in order to retank, you should extract the Liquid files."+"<br>"+"" +
				  "4) Click Extract to start extracting."+"</html>");
        
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = 3;
        c.insets = new Insets(10,10,10,10); //padding
        pane.add(labelInfo, c);
        
        //TextBox
        textTankPath = new JTextField(45);
        textTankPath.setText(conf.getProperty("TankChosen"));
        c.ipady = 0;       
        c.gridx = 0;       
        c.gridwidth = 2;   
        c.gridy = 1;               
        pane.add(textTankPath, c);
        
        //Select Button
        buttonSelectTank = new JButton("Select Tank to extract ...");
        buttonSelectTank.addActionListener(new ActionListener() { 
        	public void actionPerformed(ActionEvent e) {    
        		try{
        		showTankChooser();
        		}catch (Exception ex)
				{
        			ex.printStackTrace();
				}
        	}
        });
        
        c.ipady = 5;      //make this component tall
        c.weightx = 0.0;
        c.gridwidth = 1;
        c.gridx = 2;
        c.gridy = 1;
        pane.add(buttonSelectTank, c);

        //TextBox
        textDirectoryPath = new JTextField(45);
        textDirectoryPath.setText(conf.getProperty("ExtractChosen"));
        c.ipady = 0;      
        c.gridx = 0;       
        c.gridwidth = 2;   
        c.gridy = 2;               
        pane.add(textDirectoryPath, c);
        
        //Select Button
        buttonSelectDirectory = new JButton("Select extraction destination folder ...");
        buttonSelectDirectory.addActionListener(new ActionListener() { 
        	public void actionPerformed(ActionEvent e) {    
        		try {
        			showDirectoryChooser();
        		}catch (Exception ex){ex.printStackTrace();}
        	}
        });
        
        c.ipady = 5;      //make this component tall
        c.weightx = 0.0;
        c.gridwidth = 1;
        c.gridx = 2;
        c.gridy = 2;
        pane.add(buttonSelectDirectory, c);
        
        //Extract Button
        buttonExtract = new JButton("Extract");
        buttonExtract.addActionListener(new ActionListener() { 
        	public void actionPerformed(ActionEvent e) {    
        		extractTank();
        	}
        });
        
        c.ipady = 5;      //make this component tall
        c.weightx = 0.0;
        c.gridwidth = 2;
        c.gridx = 0;
        c.gridy = 3;
        pane.add(buttonExtract, c);
        
        //Liquid Files Option
        checkLiquid = new JCheckBox("LiquidOption");
        checkLiquid.setText("Extract Liquid Files");
        checkLiquid.addActionListener(new ActionListener() { 
        	public void actionPerformed(ActionEvent e) {    
        		
        	}
        });
        
        c.fill = GridBagConstraints.CENTER;
        c.weightx = 0.0;
        c.gridwidth = 1;
        c.gridx = 2;
        c.gridy = 3;
        pane.add(checkLiquid, c);
        
        //Progress Bar
        progressBar = new JProgressBar();
        progressBar.setValue(0);
        progressBar.setStringPainted(true); //get space for the string
        progressBar.setString(WAITING_MESSAGE);          //but don't paint it
        
        c.gridx = 0;
        c.gridy = 4;
        c.gridwidth = 1; 
        c.ipadx = 200;
        pane.add(progressBar, c);
        
        //Progress Status
        progressStatus = new JTextField(25);
        progressStatus.setText("");
        progressStatus.setBackground(pane.getBackground());
        progressStatus.setBorder(BorderFactory.createEmptyBorder());
        progressStatus.setFont(new Font(null, Font.BOLD, 12));
        progressStatus.setFocusable(false);
		
        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridx = 1;
        c.gridy = 4;
        c.gridwidth = 3;
        pane.add(progressStatus, c);
        
        //Create a timer.
        timer = new Timer(TIMER_DURATION, new ActionListener() {
        	private boolean initialized = false;
        	
        	public void actionPerformed(ActionEvent evt) {
        		
	    		if(tankReader.isStarted())
	    		{
	    			if (!initialized)
	    			{ 
	    				//This happens when the Timer is started ad the Tank Reader is started as well.
	    				//Happens only once per start
	        			setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
	        			progressBar.setString("Extracting ...");
	        			buttonExtract.setEnabled(false);
	    				
	            		if (tankReader.getLengthOfTask() == 0)
	            		{
	            			progressBar.setIndeterminate(true);
	            		}
	            		else
	            		{
	            			progressBar.setIndeterminate(false);
		            		progressBar.setMinimum(0);
		            		progressBar.setMaximum(tankReader.getLengthOfTask());
	            		}
	        			initialized = true;
	    			}
	    			else
	    			{
	    				//Update Progress Bar
		                if (!futReturn.isDone() && !futReturn.isCancelled()) 
		                {		             	
	        				progressBar.setValue(tankReader.getCurrent());
			                progressStatus.setText(tankReader.getMessage());
		                }
		                //Stop the Timer
		                else
		                {
		                	
		                    Toolkit.getDefaultToolkit().beep();			                      
		                    progressBar.setValue(progressBar.getMinimum());
		                    progressBar.setString(WAITING_MESSAGE);		                    
		            		setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		            		tankReader.resetIsStarted();
		            		buttonExtract.setEnabled(true);
		            		initialized = false;
		            		timer.stop();
		            		
		            		try{
		            			progressStatus.setText(futReturn.get());	            			
		            		}catch (Exception e){
		            			callback(e);
		            		};
		            				            		
		                }
	    			}
	            }
        	
            }
        });  
    }

    /**
     * Create the GUI and show it.  For thread safety,
     * this method should be invoked from the
     * event-dispatching thread.
     */
    private static void createAndShowGUI() throws Exception {
        //Make sure we have nice window decorations.
        JFrame.setDefaultLookAndFeelDecorated(true);

        //Create and set up the window.
        JFrame frame = new JFrame("Tank Extractor");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        //Create and set up the content pane.
        JComponent newContentPane = new TankGUI();
        newContentPane.setOpaque(true); //content panes must be opaque
        frame.setContentPane(newContentPane);
        
        //Center frame
        frame.pack();
        Dimension frameSize = frame.getSize();
        Dimension screenSize = frame.getToolkit().getScreenSize();
        frame.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
        
        //Display the window.    
        frame.setVisible(true);

    }
    
    private void showTankChooser() throws Exception{
    	JFileChooser fc;
    	
//    	Create a file chooser
    	if (conf.getProperty("TankChosen") == "" )
    	{
    		 fc = new JFileChooser();
    	}
    	else
    	{
    		 fc = new JFileChooser(conf.getProperty("TankChosen"));
    	}
    	
    	fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
///Not used anymore. Too many different extensions
//fc.setFileFilter( new AllTanksAndDirectoriesGUIFilter());
    	fc.setDialogTitle("Select a Tank File");
    		
//    	In response to a button click:
    	int returnVal = fc.showOpenDialog(frame);
    	
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            textTankPath.setText(file.getPath());
            conf.setProperty("TankChosen", file.getPath());
            conf.saveConfigFile(); 
            //Put the default to extract Liquid Files for Maps
            if (new AllMapsFilter().accept(file))
            {
            	checkLiquid.setSelected(true);
            }
        } else {
           //Command was cancelled
        }     	
    }
    
    private void showDirectoryChooser() throws Exception{
    	
//    	Create a file chooser
    	JFileChooser fc;
    	
//    	Create a file chooser
    	if (conf.getProperty("ExtractChosen") == "" )
    	{
    		 fc = new JFileChooser();
    	}
    	else
    	{
    		 fc = new JFileChooser(conf.getProperty("ExtractChosen"));
    	}
    	fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
    	fc.setDialogTitle("Select a Tank File");
    		
//    	In response to a button click:
    	int returnVal = fc.showOpenDialog(frame);
    	
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            textDirectoryPath.setText(file.getAbsolutePath());
            conf.setProperty("ExtractChosen", file.getAbsolutePath());
            conf.saveConfigFile(); 
        } else {
           //Command was cancelled
        }     	
    }
    
    private void extractTank(){ 
		
//		Input, Output, Extract, Decompress
    	tankReader = new TankReader(textTankPath.getText(), textDirectoryPath.getText(), this, true, false, checkLiquid.isSelected());
		//Starts the tank reading on a low priority thread
    	ExecutorService exec = Executors.newSingleThreadExecutor(new LowPriorityThreadFactory());
		futReturn = exec.submit(tankReader);

	    timer.start();
    }
    
    public static void main(String[] args) {
        //Schedule a job for the event-dispatching thread:
        //creating and showing this application's GUI.
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                try {
            	createAndShowGUI();
                }
                catch(Exception e)
				{
                	e.printStackTrace();
				}
            }
        });
    }
    
    public void actionPerformed(ActionEvent evt) {

    }
    
    //Method from ITankCaller
    public void callback(Exception e)
    {
    	String errorMessage;
    	String titleMessage;
    	if (e instanceof InvalidDirectoryException || e instanceof InvalidFileException || e instanceof InvalidTankException)
    	{
    		errorMessage = e.getMessage();
    		titleMessage = "Error";
    	}
    	else
    	{
    		ByteArrayOutputStream error = new ByteArrayOutputStream();
			PrintStream out  = new PrintStream (error);
			e.printStackTrace(out);
			errorMessage = error.toString();
			titleMessage = "Unexpected Error";
    	}
    	
    	Object[] possibilities = {"Close", "Close and Copy to Clipboard"};
    	int choice = JOptionPane.showOptionDialog(null,
        		errorMessage,
        		titleMessage,
        		JOptionPane.ERROR_MESSAGE, 
        		JOptionPane.YES_NO_OPTION, 
        		null, 
        		possibilities, 
        		possibilities[0]);
        				
        if (choice == JOptionPane.NO_OPTION) {
            //Copy to Clipboard
        	Clipboard clipboard = getToolkit().getSystemClipboard ();
        	StringSelection s = new StringSelection(errorMessage);
        	clipboard.setContents(s, s);
        }
    }
}