/////////////////////////////////////////////////////////////////////////////
//File     :  TankHeader.java
//Author(s):  Chris (Episthene) Baillargeon
//Description: Read and Store the Tank Header

//copyright (C) 2005 Chris (Episthene) Baillargeon

//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//http://www.gnu.org/licenses/gpl.html

//----------------------------------------------------------------------------
//$Revision: 1.0               $Date: 2005/10/06  
//----------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
package epi_extractor;

import java.nio.channels.*;
import java.nio.*;
import java.io.*;
import epi_tools.*;
import java.math.*;

public class TankHeader {

	private ByteBuffer ProductId = ByteBuffer.allocate(4);
	private ByteBuffer TankId = ByteBuffer.allocate(4);
	private ByteBuffer HeaderVersion = ByteBuffer.allocate(4);
	private ByteBuffer DirectoryOffset = ByteBuffer.allocate(4);
	private ByteBuffer FileOffset = ByteBuffer.allocate(4);
	private ByteBuffer IndexSize = ByteBuffer.allocate(4);
	private ByteBuffer DataOffset = ByteBuffer.allocate(4);
	private ByteBuffer ProductVersion = ByteBuffer.allocate(12);
	private ByteBuffer MinimumVersion = ByteBuffer.allocate(12);
	private ByteBuffer Priority = ByteBuffer.allocate(4);
	private ByteBuffer Flags = ByteBuffer.allocate(4);
	private ByteBuffer CreatorId = ByteBuffer.allocate(4);
	private ByteBuffer GUID = ByteBuffer.allocate(36);
	private ByteBuffer IndexCRC32 = ByteBuffer.allocate(4);
	private ByteBuffer DataCRC32 = ByteBuffer.allocate(4);
	private ByteBuffer TimeStamp = ByteBuffer.allocate(16);
	private ByteBuffer Copyright = ByteBuffer.allocate(180);
	private ByteBuffer BuildText = ByteBuffer.allocate(200);
	private ByteBuffer TitleText = ByteBuffer.allocate(200);
	private ByteBuffer AuthorText = ByteBuffer.allocate(100);
	
	public TankHeader(FileChannel inChannel) throws IOException
	{
		inChannel.read(ProductId, 0);
		inChannel.read(TankId, 4);
		inChannel.read(HeaderVersion, 8);
		inChannel.read(DirectoryOffset, 12);
		inChannel.read(FileOffset, 16);
		inChannel.read(IndexSize, 20);
		inChannel.read(DataOffset, 24);
		inChannel.read(ProductVersion, 28);
		inChannel.read(MinimumVersion, 40);
		inChannel.read(Priority, 52);
		inChannel.read(Flags, 56);
		inChannel.read(CreatorId, 60);
		inChannel.read(GUID, 64);
		inChannel.read(IndexCRC32, 100);
		inChannel.read(DataCRC32, 104);
		inChannel.read(TimeStamp, 108);
		inChannel.read(Copyright, 124);
		inChannel.read(BuildText, 304);
		inChannel.read(TitleText, 504);
		inChannel.read(AuthorText, 704);
		
//System.out.println(this.toString());

	}
	
	public String toString()
	{
		String output = "";
		try
		{
			output = output + "Product ID: " + new String( ProductId.array(), "US-ASCII" ) + "\n";
			output = output + "Tank ID: " +  new String(TankId.array(), "US-ASCII" ) + "\n";
			output = output + "Version ID: " +  Hexadecimal.valueOf(HeaderVersion.array()) + "\n";
			output = output + "Directory Offset: " +  Integer.reverseBytes(new BigInteger(DirectoryOffset.array()).intValue()) + "\n";
			output = output + "File Offset: " +  Integer.reverseBytes(new BigInteger(FileOffset.array()).intValue()) + "\n";
			output = output + "Index Size: " +  Integer.reverseBytes(new BigInteger(IndexSize.array()).intValue()) + "\n";
			output = output + "Data Offset: " +  Integer.reverseBytes(new BigInteger(DataOffset.array()).intValue()) + "\n";
			output = output + "Product Version: " +  Hexadecimal.valueOf(ProductVersion.array()) + "\n";
			output = output + "Minimum Version: " +  Hexadecimal.valueOf(MinimumVersion.array()) + "\n";
			output = output + "Priority: " +  Hexadecimal.valueOf(Priority.array()) + "\n";
			output = output + "Flags: " +  Hexadecimal.valueOf(Flags.array()) + "\n";
			output = output + "Creator ID: " +  new String(CreatorId.array(), "US-ASCII" ) + "\n";
			output = output + "GUID: " +  Hexadecimal.valueOf(GUID.array()) + "\n";
			output = output + "IndexCRC32: " +  Hexadecimal.valueOf(IndexCRC32.array()) + "\n";
			output = output + "DataCRC32: " +  Hexadecimal.valueOf(DataCRC32.array()) + "\n";
			output = output + "TimeStamp: " +  Hexadecimal.valueOf(TimeStamp.array()) + "\n";
			output = output + "Copyright: " +  new String(Copyright.array(), "US-ASCII" ) + "\n";
			output = output + "Build: " +  new String(BuildText.array(), "US-ASCII" ) + "\n";
			output = output + "Title: " +  new String(TitleText.array(), "US-ASCII" ) + "\n";
			output = output + "Author: " +  new String(AuthorText.array(), "US-ASCII" ) + "\n";
		
		}catch (Exception e)
			{
			System.out.println(e.toString());
			}
		return output;
	}
	
	public int getIndexSize()
	{
		return Integer.reverseBytes(new BigInteger(IndexSize.array()).intValue());
	}
	public int getDirectoryOffset()
	{
		return Integer.reverseBytes(new BigInteger(DirectoryOffset.array()).intValue());
	}
	public int getFileOffset()
	{
		return Integer.reverseBytes(new BigInteger(FileOffset.array()).intValue());
	}
	public int getDataOffset()
	{
		return Integer.reverseBytes(new BigInteger(DataOffset.array()).intValue());
	}
	public String getTankId()
	{
		try{
			return Epi_Tools.getByteString(TankId.array());
		}catch(Exception e){return "";}
		 
	}
}
